import {
    CollectionViewer,
    DataSource
} from '@angular/cdk/collections';

import { Observable } from 'rxjs/Observable';
import { BehaviorSubject } from 'rxjs/BehaviorSubject';
import { of } from 'rxjs/observable/of';

import {
    catchError,
    finalize
} from 'rxjs/operators';

import { QuotesService } from './quotes.service';

import {
    Quotes,
    Quote
} from '../models/quotes';


export class QuotesDataSource implements DataSource<Quote> {
    private quotesSubject = new BehaviorSubject<Quote[]>([]);
    private countSubject = new BehaviorSubject<number>(0);
    private loadingSubject = new BehaviorSubject<boolean>(false);

    public count$ = this.countSubject.asObservable();
    public loading$ = this.loadingSubject.asObservable();

    constructor(private quotesService: QuotesService) { }

    loadQuotes(filter: string,
               sortColumn: string,
               sortDirection: string,
               pageIndex: number,
               pageSize: number) {

        this.loadingSubject.next(true);

        this.quotesService.findQuotes(filter, sortColumn, sortDirection,
                                      pageIndex, pageSize)
            .pipe(
                catchError(() => of([])),
                finalize(() => this.loadingSubject.next(false))
            )
            .subscribe(quotes => {
                this.quotesSubject.next((<Quotes>quotes).quotes);
                this.countSubject.next((<Quotes>quotes).totalCount)
            });
    }

    connect(collectionViewer: CollectionViewer): Observable<Quote[]> {
        console.log('Connecting data source');
        return this.quotesSubject.asObservable();
    }

    disconnect(collectionViewer: CollectionViewer): void {
        this.quotesSubject.complete();
        this.countSubject.complete();
        this.loadingSubject.complete();
    }
}
